# -*- coding: utf-8 -*-
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


"""
Test case for ContentDispatcher installation
"""

__docformat__ = 'restructuredtext'

from StringIO import StringIO

# Products imports
from Products.ContentDispatcher.tests.base import  BaseTestCase, DummyContent,\
    schema

from Products.ContentDispatcher.plugins import ZexpImporter, FssImporter, \
    ZexpExporter, FssExporter
# To serialize object
from Products.ContentDispatcher.utils import serializeObject,\
    unserialiseObject

#CMF
from Products.CMFCore.utils import getToolByName
from Products.ContentDispatcher.content import ZexpPluginData

# interface for get adaptater
from Products.ContentDispatcher.interface import IPluginExporter
from Products.ContentDispatcher.content import ContentDispatcher

# transaction
from transaction import commit

#Archtypes import
from Products.Archetypes.tests.utils import mkDummyInContext
from Products.Archetypes.config import UUID_ATTR

from random import random

def createContent(context,oid,text1,text2):
    """ 
    create content to test import fss
    the structure
    context/dummy
    context/dummy/dummy
    
    dummy is a at folder wich have a file with text1 for the first one
    and text2 for the second one
    """
    
    f1=mkDummyInContext(DummyContent,
                        oid=oid,
                        context=context,
                        schema = schema)

    ## be careful uid must be unique for fss
    updateUid(f1)
    f1.update(**{'file':StringIO(text1)})
    
    f2=mkDummyInContext(DummyContent,
                        oid=oid,
                        context=f1,
                        schema = schema)
    updateUid(f2)
    f2.update(**{'file':StringIO(text2)})
    

def updateUid(context):
    setattr(context,UUID_ATTR,'%s_%i' % (context.getId(),
                                         int(random() *1000000)))




class TestImport(BaseTestCase):
    """
    test  product installation
    """

    def testImportZexp(self):
        """
        test import via zexp, create an content strucure and import her to
        an another strucure
        """

        self.setRoles(['Manager'])
       
        portal = self.portal
        portal.invokeFactory(id='folderimport',
                             type_name='Folder',
                             )
        portal.folderimport.invokeFactory(id='file',
                                        type_name='File')
        
        portal.folderimport.invokeFactory(id='document',
                                        type_name='Document')
        ## we must commit 
        commit()
        
        ## create the data plugin exporter
        context = self.portal.folderimport
        zexpexporter = ZexpExporter(context)
        zexpdata = zexpexporter.exportData()

        ## we get zexpimporter
        importer = IPluginExporter(zexpdata)

        ## and we import to the folder import
        
        portal.invokeFactory(id='content',
                             type_name='Folder',
                             )
        importer.importData(portal.content)
        self.failUnless(hasattr(portal.content,'folderimport'))
        
        
    def testExportZexp(self):
        """
        test export fss
        """
        self.setRoles(['Manager'])
       
        portal = self.portal
        # create the first structure
        # portal/dummy/dummy
        inittext = 'this the initial text'
        createContent(portal,
                      'dummy',
                      inittext,
                      inittext )
        
        portal.invokeFactory(id='content',
                             type_name='Folder',
                             )
        
        # create the first structure
        # portal/content/dummy/dummy
        text='this the a text'
        createContent(portal.content,
                      'dummy',
                      text,
                      text )
        ## now we test the two structure
        
        self.failUnless(portal.dummy.getFile().getData() == inittext)
        self.failUnless(portal.dummy.dummy.getFile().getData() == inittext)
        
        self.failUnless(portal.content.dummy.getFile().getData() == text)
        self.failUnless(portal.content.dummy.dummy.getFile().getData() == text)
        
        ## and now we do an import of the first structure to the second one
        fssexporter = FssExporter(portal.dummy)
        data =  fssexporter.exportData()
        
        importer = IPluginExporter(data)
        importer.importData(portal.content)

        ## test if export succeed
        self.failUnless(portal.content.dummy.getFile().getData() == inittext)
        self.failUnless(portal.content.dummy.dummy.getFile().getData() == \
                            inittext)
        
        
        
        
    def testContentDispatcher(self):
        """
        test serialisation of an import
        """
        self.setRoles(['Manager'])
        portal = self.portal
        dtool = getToolByName(portal,'content_dispatcher_tool')
        ## we register pluggins
        dtool.registerPlugin('zexp', ZexpExporter, 0)
        dtool.registerPlugin('fss', FssExporter, 10)

        ## create content 
        # create the first structure
        # portal/dummy/dummy
        inittext = 'this the initial text'
        createContent(portal,
                      'dummy2',
                      inittext,
                      inittext )
        
        portal.invokeFactory(id='finalcontent',
                             type_name='Folder',
                             )
        commit()
        
        todispatch=ContentDispatcher(portal.dummy2)
        tosend=serializeObject(todispatch)
        todispatch=unserialiseObject(tosend)
        ## and now we dispatch content to portal.content
        todispatch(portal.finalcontent)
        ## we verify that we have good results
        

        ## test if export succeed
        self.failUnless(portal.finalcontent.dummy2.getFile().getData()\
                            == \
                            inittext)
        self.failUnless(portal.finalcontent.dummy2.dummy2.getFile().getData()\
                            == \
                            inittext)
        
        

        





def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestImport))
    return suite
